"{ Package: 'jn:refactoring_custom' }"

RefactoryBuilder subclass:#CustomRefactoryBuilder
	instanceVariableNames:'changeManager formatter'
	classVariableNames:''
	poolDictionaries:''
	category:'Interface-Refactoring-Custom'
!

!CustomRefactoryBuilder class methodsFor:'documentation'!

documentation
"
    Encapsulates performing refactoring changes on the source code within single object. 
    Single refactorings are stored as change objects which represens changes in the source code.

    [author:]
        Jakub Nesveda <nesvejak@fit.cvut.cz> 

"
! !

!CustomRefactoryBuilder methodsFor:'accessing'!

change
    "Returns single code change from collected code changes."

    | change |

    changes isEmpty ifTrue: [ 
        ^ nil
    ].

    change := CompositeRefactoryChange new.

    changes do: [ :codeChange | 
        change addChange: codeChange
    ].

    ^ change

    "Created: / 15-08-2014 / 00:45:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 17-08-2014 / 17:16:38 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

changeManager
    ^ changeManager
!

changeManager:something
    changeManager := something.
!

formatter
    ^ formatter
!

formatter: aFormatter
    formatter := aFormatter.

    "Modified (format): / 31-08-2014 / 17:12:26 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

parser

    ^ RBParser

    "Created: / 24-08-2014 / 23:36:55 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

searcher

    ^ ParseTreeSearcher new

    "Created: / 16-08-2014 / 22:13:25 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomRefactoryBuilder methodsFor:'compiling'!

execute
    "
    Most likely as compile method, but without undo change
    in global RefactoryChangeManager
    (true for CustomLocalChangeManager changeManager implementation)
    "

    changeManager performChange: self change

    "Created: / 15-08-2014 / 00:45:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

redoChanges
    "redo all changes made by execute method"

    changeManager redoChanges

    "Created: / 15-08-2014 / 00:46:47 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomRefactoryBuilder methodsFor:'initialization'!

initialize
    "Invoked when a new instance is created."

    super initialize.
    changeManager := CustomLocalChangeManager new.
    formatter := CustomRBLocalSourceCodeFormatter new.

    "Created: / 15-08-2014 / 00:42:39 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 31-08-2014 / 17:11:21 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomRefactoryBuilder methodsFor:'parsing'!

parseExpression: aSourceCode

    ^ self parser parseExpression: aSourceCode onError: [ :string :pos |
        self parser parseMethod: aSourceCode onError: [ :string :pos |
            self error: 'Could not parse ', string, ' at pos ', pos asString
        ]
    ]

    "Created: / 25-08-2014 / 22:34:49 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

parseMethod: aSourceCode

    ^ self parser parseMethod: aSourceCode onError: [ :string :pos |
        self parser parseExpression: aSourceCode onError: [ :string :pos |
            self error: 'Could not parse ', string, ' at pos ', pos asString
        ]
    ]

    "Created: / 25-08-2014 / 22:35:32 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomRefactoryBuilder methodsFor:'refactory-changes'!

executeReplace: searchPattern with: rewritePattern inCodeSelection: aCodeSelection

    | newTree change parseTree selectedParseTree newSelectedParseTree |

    selectedParseTree := self parseExpression: aCodeSelection selectedSourceCode.

    "/ Transcript showCR: 'SS: ', selectedParseTree asString.  
    "/ Transcript showCR: 'SC: ', aCodeSelection selectedSourceCode.  

    newSelectedParseTree := (CustomParseTreeRewriter new)
        replace: searchPattern with: rewritePattern;
        executeTree: selectedParseTree;
        tree.

    "/ Transcript showCR: 'NSS: ', newSelectedParseTree asString.

    parseTree := self parseMethod: aCodeSelection currentSourceCode.
    newTree := CustomParseTreeRewriter 
        replace: aCodeSelection selectedSourceCode
        with: (formatter formatParseTree: newSelectedParseTree)
        in: parseTree
        onInterval: aCodeSelection selectedInterval.

    change := InteractiveAddMethodChange
        compile: (formatter formatParseTree: (self parseMethod: newTree newSource))
        in: aCodeSelection selectedMethod mclass 
        classified: aCodeSelection selectedMethod category.

    changes add: change

    "Created: / 24-08-2014 / 10:24:52 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 31-08-2014 / 17:17:01 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

executeReplace: searchPattern with: rewritePattern inMethod: aMethod
    "Executes replace in a method source and then creates a method change"

    | newTree change |

    newTree := (CustomParseTreeRewriter new)
        replace: searchPattern with: rewritePattern;
        executeTree: aMethod parseTree;
        tree.

    change := InteractiveAddMethodChange
        compile: (formatter formatParseTree: (self parseMethod: newTree newSource))
        in: aMethod mclass 
        classified: aMethod category.

    changes add: change

    "Created: / 17-08-2014 / 18:45:20 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 31-08-2014 / 17:41:35 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

replace:searchPattern with:rewritePattern inCodeSelections:aCodeSelectionCollection 
    "Searches for given pattern in methods source code and if source code matches then executes replacement"
    
    self 
        search:searchPattern
        inCodeSelections:aCodeSelectionCollection
        withResultDo:[:codeSelection | 
            self 
                executeReplace:searchPattern
                with:rewritePattern
                inCodeSelection:codeSelection
        ]

    "Created: / 24-08-2014 / 10:10:40 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 26-08-2014 / 23:09:26 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

replace: searchPattern with: rewritePattern inContext: aCustomContext
    "Searches for given pattern in methods source code and if source code matches then executes replacement"

    self 
        search: searchPattern 
        inContext: aCustomContext 
        withResultDo: [ :method |
            self executeReplace: searchPattern with: rewritePattern inMethod: method 
        ]

    "Created: / 16-08-2014 / 19:15:10 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified (comment): / 17-08-2014 / 19:30:11 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomRefactoryBuilder methodsFor:'searching'!

search: searchPattern inClass: aClass withResultDo: aBlock
    | methods |

    methods := OrderedCollection new.

    methods addAll: aClass theNonMetaclass methodDictionary values.    
    methods addAll: aClass theMetaclass methodDictionary values.

    self search: searchPattern inMethods: methods withResultDo: aBlock

    "Created: / 17-08-2014 / 13:15:03 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 17-08-2014 / 17:18:27 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

search: searchPattern inClassCategories: aCategoryCollection withResultDo: aBlock

    "To be implemented"

    "Created: / 17-08-2014 / 16:21:23 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

search: searchPattern inClasses: aClassCollection withResultDo: aBlock

    aClassCollection do:[ :class | 
        self search: searchPattern inClass: class withResultDo: aBlock
    ].

    "Created: / 17-08-2014 / 16:21:22 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

search:searchPattern inCodeSelections:aCodeSelectionCollection withResultDo:aBlock 

    aCodeSelectionCollection do:[ :codeSelection | 
        | parseTree selectedCode |

        selectedCode := codeSelection selectedSourceCode.
        parseTree := self parseExpression: selectedCode.
        "/ Transcript showCR: 'PP', parseTree asString.  
        (self searcher)
            matches: searchPattern do: [ :aNode :answer |
                aBlock value: codeSelection  
            ];
            executeTree: parseTree
    ].

    "Created: / 21-08-2014 / 21:22:13 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 25-08-2014 / 22:36:07 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

search: searchPattern inContext: aCustomContext withResultDo: aBlock
    "Searches for source code pattern in whole context which contains code, methods, classes... "

    aCustomContext selectedCodes notNil ifTrue: [ 
        self 
              search:searchPattern
              inCodeSelections:aCustomContext selectedCodes
              withResultDo:aBlock
    ].

    aCustomContext selectedMethods notNil ifTrue: [ 
        self 
            search: searchPattern 
            inMethods: aCustomContext selectedMethods
            withResultDo: aBlock
    ].

    aCustomContext selectedMethodProtocols notNil ifTrue: [ 
        self 
            search: searchPattern 
            inMethodProtocols: aCustomContext selectedMethodProtocols
            withResultDo: aBlock
    ].

    aCustomContext selectedClasses notNil ifTrue: [ 
        self 
            search: searchPattern 
            inClasses: aCustomContext selectedClasses
            withResultDo: aBlock
    ].

    aCustomContext selectedClassCategories notNil ifTrue: [ 
        self 
            search: searchPattern 
            inClassCategories: aCustomContext selectedClassCategories
            withResultDo: aBlock
    ].

    aCustomContext selectedPackages notNil ifTrue: [ 
        self 
            search: searchPattern 
            inPackages: aCustomContext selectedPackages
            withResultDo: aBlock
    ].

    "Created: / 17-08-2014 / 16:21:24 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 21-08-2014 / 21:22:03 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

search: searchPattern inMethod: aMethod withResultDo: aBlock
    | parseTree |

    parseTree := aMethod parseTree.    

    parseTree isNil ifTrue: [ 
        self error: 'Cannot retrieve parseTree for method: ', aMethod asString.
    ]
    ifFalse: [
        (self searcher)
            matches: searchPattern do: [ :aNode :answer |
                aBlock value: aMethod  
            ];
            executeTree: parseTree
    ].

    "Created: / 16-08-2014 / 22:27:35 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

search: searchPattern inMethodProtocols: aProtocolCollection withResultDo: aBlock

    "To be implemented"

    "Created: / 17-08-2014 / 16:21:21 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

search: searchPattern inMethods: aMethodCollection withResultDo: aBlock

    aMethodCollection do: [ :method | 
        self search: searchPattern inMethod: method withResultDo: aBlock
    ].

    "Created: / 17-08-2014 / 11:56:23 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
    "Modified: / 17-08-2014 / 20:35:16 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
!

search: searchPattern inPackages: aPackageCollection withResultDo: aBlock

    "To be implemented"

    "Created: / 17-08-2014 / 16:21:24 / Jakub Nesveda <nesvejak@fit.cvut.cz>"
! !

!CustomRefactoryBuilder class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

